%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Optimal Distributed Control (ODC) Problem
%
% Designed for Determininstic, Stochastic, Infinite-Horizon and Infinite-Horizon Problems
%---------------------------------------------
% Deterministic ODC:
%
% This solver solves an SDP relaxation for the following deterministic ODC:
%
% minimize 
%        sum(x[t]'Qx[t]+u[t]'Ru[t])+alpha*trace(K*K')   for t=0,...,p
%
% subject to 
%         x[t+1]=Ax[t]+Bu[t]
%         y[t]=Cx[t]               for t=0,...,p
%
%         trace(K*K')<=1/beta
%
% where x[t], y[t] and u[t] denote the state, output and input of the system
% at time t, and (.)' represents the transpose operator. Also:
%
% alpha = A nonnegtaive number used to penalize the controller gain as "alpha*trace(K*K')"
%
% and
%
% beta = A nonnegtaive number used to penalize  the controller gain as "trace(K*K')<=1/beta"
%
% The aim is to design a static controller K such that u[t]=Ky[t], where the strcture of $K$ must satisfy 
% the sparsity pattern sign(K)=L for a given binary matrix L. 
%
% The above problem is called finite-horizon ODC if "p" is finite and is
% called infinite-horizon deterministic ODC whenever "p" is infinity.
%
%----------------------------------------------
% Stochastic ODC:
%
% This solver also solves an SDP relaxation for the following stochastic ODC:
%
% minimize 
%        lim(E{(x[t]'Qx[t]+u[t]'Ru[t])})+alpha*trace(K*K')   as t ---> infinity
%
% subject to 
%         x[t+1]=Ax[t]+Bu[t]+Ed[t]
%         y[t]=Cx[t]+Fv[t]            for t=0,...,p
%         trace(K*K')<=1/beta
%
% where d[t] and v[t] denote the input disturbance and measurment noise.
%
% The goal is to design a static controller K such that u[t]=Ky[t] and
% sign(K)=L for a given binary matrix L.

%% Please Provide the System Dynamics and the Method:
% A, B, C, Q, R, p, Sigma_noise, Sigma_disturbance, alpha, beta, and L are the input of the solver.
% When you run the solver, you will be asked to choose the type of the ODC problem, the recovery method, the SDP solver, and the penalty coeffciient "epsilon".
% Please put your data in the file "System_Description".
% Two example systems for mass spring and power networks are provided you can run the code for them too.
% To reduce the computational time, we recommend the use of the SDP solver "MOSEK" (this needs a CVX professional).
% To improve the quality of the solution, you may need to finetune the penalty coefficients of the objective function in the m.files of the the indirect
% recovery method.



%%

% Important: The following line should be changed to Example_PowerSystem or Example_MassSpring if you would like to try power system or mass spring simulations.
System_Description

disp('You need to specify a 1*4 vector "M" with the following characteristics:')


disp(sprintf ('\n M(1): enter 0 for finite-horizon ODC, 1 for infinite-horizon ODC, and 2 for stochstic ODC \n M(2): enter 0 for Direct method and 1 for Indirect Method \n M(3): enter 0 for MOSEK, 1 for SDP3 and 2 for SEDUMI \n M(4): enter a nonnegative number epsilon for trace penalty.') )
disp(sprintf ('\n (Note that if you would like to use the direct recovery method, beta must be zero.)'))

options=input('\n Please enter the vector "M" (e.g., [2 1 1 0]):  ');

[Upper_Bound,Lower_Bound,K,W] = Main_ODC_Solver(A,B,C,X0,L,Q,R,p,Sigma_disturbance,Sigma_noise,options,alpha,beta);









